# from geojson_pydantic.types import BBox
#
# _ch_lon = [45.7553, 47.6203]
# _ch_lat = [5.7127, 10.5796]
#
# CH: BBox = (_ch_lon[0], _ch_lat[0], _ch_lon[1], _ch_lat[1])

# https://gist.github.com/graydon/11198540
