# some coordiantes are not correct and are hardcoded here since the coordinates are used to uniquely identify a hut.
CORRECTIONS: dict[int, tuple[float, float]] = {
    # id: (lat     , lon    )   # hut name
    936745007: (45.95379, 7.15546),  # Cabane de la Tsissette
}
