# some coordiantes are not correct and are hardcoded here since the coordinates are used to uniquely identify a hut.
CORRECTIONS: dict[int, tuple[float, float]] = {
    # id: (lat     , lon    )   # hut name
    2414: (46.52605, 8.17309),  # Oberaarjochhuette
    2980: (46.38300, 7.46782),  # Wildstrubelhuette
    3418: (46.48304, 7.12020),  # La Cabane des Choucas
    403: (46.30683, 6.74356),  # Refuge d'Ubine
    3605: (46.47469, 6.91070),  # le-Grillet
    419: (46.33056, 6.76587),  # Refuge de Bise
    2265: (45.98630, 7.63479),  # Lonza biwak
    5170: (46.2174989, 8.4489458),  # Bivacco-Campolatte-Baitin-dul-Peurat
    2336: (45.93388, 7.70813),  # Rifugio Testa Grigia Guide del Cervino
    2463: (46.15540, 7.27787),  # Cabane de Balavaux
    239: (45.89486, 6.98097),  # Refuge de Leschaux
    2382: (46.13942, 7.98927),  # Hohsaashütte
    2464: (46.07422, 7.86581),  # Mischabeljoch biwak
    3638: (46.05932, 7.06211),  # Cabane de Bovinette
}
