# from hut_services.core.utils import GPSConverter
#
#
# def test_wgs84_to_lv03_swiss() -> None:
#    converter = GPSConverter()
#
#    # Coordinates
#    wgs84 = [46.95108, 7.438637, 0]
#    # Convert WGS84 to LV03 coordinates
#    lv03 = converter.WGS84toLV03(wgs84[0], wgs84[1], wgs84[2])
#
#    print("WGS84: ")
#    print(wgs84)
#    print("LV03: ")
#    print(lv03)
#    assert lv03 == (600000.3294154685, 199999.70729839153, -49.554241824400016)
